#include <stdio.h>
#include <reg51.h>
#include <sys51.h>
#include <irq52.h>

#define mt8888  P1
#define rs0     P0_B7
#define re      P0_B6 //1=terhubung
#define hk      P0_B5 //diangkat = 1
#define ri      P0_B4
#define int0    P0_B0
#define cs      P0_B1
#define wr      P0_B2
#define rd      P0_B3
#define to      P2_B0
#define dtmf1   0x0D  //TOUT,DTMF,IRQ,BURST(50ms)
#define dtmf2   0x0F  //TOUT,CP,IRQ,BURST(100ms)
#define cp      0x0e
#define tonemd  0x1c
#define toneenb 0x05 
#define tonedis 0x04

near char LP,F01,F02, init_mode;

IRQ_VECTOR(isr_tim0,TIMER0)

void isr_tim0(void) interrupt{
    LP++;
    TH0=0x4c;
    TL0=0;
}

void initialize232(void){
    TMOD=TMOD&0x0f;
    TMOD=TMOD|0x20;//mode
    SCON=0x50;
    PCON=PCON&0x7f;
    TH1=0xfd;
    TL1=0xfd;
    EA=1;
    ET1=0;
    TR1=1;
}

void init_timer0_50ms(void){
    TMOD=TMOD&0xf0;
    TMOD=TMOD|1;
    TH0=0x4c;
    TL0=0x0;
    TF0=0;
    TR0=0;
}

void sendserial(char A){
    SBUF=A;
    while (TI==0){;}
    TI=0;
}

char receiveserial(void){
    while (RI==0){;}
    RI=0;
    return SBUF;
}

void sendstring(char *const number){
    while(*number!=0){ 
        sendserial (*number);
        number++;       
    }
} 

char read_stat(void){
    char out;
    rs0=1;
    cs=0;
    rd=0;
    mt8888=0xff;
    _wait_ms(1);
    out=mt8888;
    out&=0x0f;
    rd=1;
    cs=1;
    rs0=0;
    return out;
}

void write_cr(char input){
    rs0=1;
    cs=0;
    mt8888=input;
    wr=0;
    wr=1;
    cs=1;
    rs0=0;
}

void reset_8888(void){
    read_stat();
    write_cr(0x00);
    write_cr(0x00);
    write_cr(0x08);
    write_cr(0x00);
    read_stat();
}

void init_8888(char mode){
    char mode_swap;
    mode_swap=mode & 0x08;
    write_cr(mode);
    if (mode_swap==0x08){
        write_cr(mode>>=4);
    }
}

char rooh(void){
    char ring, end=0;
    ri=0;
    hk=0;
    while (end==0){
        _nop_();
        if (hk==1){
            _wait_ms(30);
            if (hk==1){
                ring=0;
                end=1;
            }
        }
        if (end==0){
            if (ri==0){
                _wait_ms(50);
                if (ri==0){
                    ring=1;
                    end=1;
                }
            }
        }
    } 
    return ring;      
}
 
char ring_det2 (char number){
    char timeout,ringexist=0;
    LP=0;
    timeout=180;
    init_timer0_50ms();
    TR0=1;
    while(1){
        if (ri==0){
            TR0=0;
            _wait_ms(2000);
            number--;
            if (number==0){
                ringexist=1;
                return ringexist;
            }
            else {
                LP=0;
                timeout=180;
                init_timer0_50ms();
                TR0=1;
            }
        }
        else if (TF0==1){
            LP++;
            TF0=0;
            TH0=0x4c;
            TL0=0;
            if (LP==20){
                LP=0;
                timeout--;
                if (timeout==0){
                    ringexist=0;
                    return ringexist;
                }
            }
        }
    }
            
    return ringexist;
}  

void main(void){
    char mode, data;
    char *const offhook = "offhook";
    char *const ring = "ring";
    init_mode=dtmf1;
    re=1;
    wr=1;
    rd=1;
    rs0=1;
    cs=1;
    int0=1;
    hk=1;
    initialize232();  
    reset_8888();
    init_8888(init_mode);
    while(1){
        if (rooh()==1){
            sendstring(ring);
            if (ring_det2(2)==1){
                re=0;
                _wait_ms(250);
                re=1;
            }
        }
        else{
            sendstring(offhook);
            sendserial(0x0d);
        }      
    }
//out:
}
