#include <stdio.h>
#include <reg51.h>
#include <sys51.h>
#include <irq52.h>

#define mt8888  P1
#define rs0     P0_B7
#define re      P0_B6 //1=terhubung
#define hk      P0_B5 //diangkat = 1
#define ri      P0_B4
#define int0    P0_B0
#define cs      P0_B1
#define wr      P0_B2
#define rd      P0_B3
#define to      P2_B0
#define dtmf1   0x0D  //TOUT,DTMF,IRQ,BURST(50ms)
#define dtmf2   0x0F  //TOUT,CP,IRQ,BURST(100ms)
#define cp      0x0e
#define tonemd  0x1c
#define toneenb 0x05 
#define tonedis 0x04

near char LP,F01,F02, init_mode;

IRQ_VECTOR(isr_tim0,TIMER0)

void isr_tim0(void) interrupt{
    LP++;
    TH0=0x4c;
    TL0=0;
}

void initialize232(void){
    TMOD=TMOD&0x0f;
    TMOD=TMOD|0x20;//mode
    SCON=0x50;
    PCON=PCON&0x7f;
    TH1=0xfd;
    TL1=0xfd;
    EA=1;
    ET1=0;
    TR1=1;
}

void init_timer0_50ms(void){
    TMOD=TMOD&0xf0;
    TMOD=TMOD|1;
    TH0=0x4c;
    TL0=0x0;
    TF0=0;
    TR0=0;
}

void sendserial(char A){
    SBUF=A;
    while (TI==0){;}
    TI=0;
}

char receiveserial(void){
    while (RI==0){;}
    RI=0;
    return SBUF;
}

void sendstring(char *const number){
    while(*number!=0){ 
        sendserial (*number);
        number++;       
    }
} 

char read_stat(void){
    char out;
    rs0=1;
    cs=0;
    rd=0;
    mt8888=0xff;
    _wait_ms(1);
    out=mt8888;
    out&=0x0f;
    rd=1;
    cs=1;
    rs0=0;
    return out;
}

void write_dtmf(char input){
    rs0=0;
    cs=0;
    mt8888=input;
    wr=0;
    wr=1;
    cs=1;
}

void write_cr(char input){
    rs0=1;
    cs=0;
    mt8888=input;
    wr=0;
    wr=1;
    cs=1;
    rs0=0;
}

void reset_8888(void){
    read_stat();
    write_cr(0x00);
    write_cr(0x00);
    write_cr(0x08);
    write_cr(0x00);
    read_stat();
}

void init_8888(char mode){
    char mode_swap;
    mode_swap=mode & 0x08;
    write_cr(mode);
    if (mode_swap==0x08){
        write_cr(mode>>=4);
    }
}

void delay14(void){//delay 1.4 ms
    char i,j;
    for (i=0;i<6;i++){
        for (j=0;j<110;j++){}
    }   
}

char check_dial_tone(void){//0=not existed/1=existed
    char end=0,flag,i;
    init_8888(cp);
    _wait_ms(60);
    while (end==0){
        to=1;
        i=to;
        _wait_ms(30);
        if (to==i){
            end=1;
            flag=to^1;
        }
    }
    init_8888(init_mode);
    return flag;
}

void dtmf_dialing(char * const number){
    while(*number!='$'){
        if (*number=='0'){
            write_dtmf(0x0a);
            while((read_stat()&2)!=2);
            number++;
        }
        else{
            if (*number>='1'){
                if (*number<='9'){
                    write_dtmf(*number-0x30);    
                    while((read_stat()&2)!=2);
                    number++;
                }
                else
                    *number='$';
            }
            else
                *number='$';
        }
    }   
} 

char cek_cp(void){//bit 0 = busy flag / bit 1 = rb flag
    char loop=2, flag;
    F02=0;
    init_8888(cp);
    _wait_ms(60);
    init_timer0_50ms();
    to=1;
    loop=2;
    while(loop!=0){
        LP=0;
        while(1){
            if (to==1){
                _wait_ms(40);
                if (to==1){
                    LP=0;
                    TF0=1;
                    TR0=1;
                    ET0=1;
                    EA=1;
                    while(to==1){
                        if (LP>=80){
                            TR0=0;
                            EA=0;
                            ET0=0;
                            TF0=0;
                            flag=01;//connect
                            init_8888(init_mode);
                            return flag;
                        }
                    }
                    TR0=0;
                    EA=0;
                    ET0=0;
                    TF0=0;
                }
            }
            if (to==0){
                _wait_ms(40);
                if (to==0){
                    LP=0;
                    TF0=1;
                    TR0=1;
                    ET0=1;
                    EA=1;
                    while(to==0){
                        if (LP>=21){
                            TR0=0;
                            EA=0;
                            ET0=0;
                            TF0=0;
                            flag=11;//unknown
                            init_8888(init_mode);
                            return flag;
                        }
                    }
                    TR0=0;
                    EA=0;
                    ET0=0;
                    TF0=0;
                    if (LP<11)flag=00;//busy
                    else flag=10;//ringback
                    loop--;
                    if (loop==0){
                        init_8888(init_mode);
                        return flag;
                    }
                    else break;
                }
            }
        }
    }

    return flag;
}


void main(void){
    char mode, flag;
    char *const number = "08123142799$";
    char *const ringback = "RingBack";
    char *const busy = "Busy";
    char *const connect = "Connect";
    char *const unknown = "Unknown";
    char *const onhook = "Onhook";
    init_mode=dtmf1;
    re=1;
    wr=1;
    rd=1;
    rs0=1;
    cs=1;
    int0=1;
    hk=1;
    initialize232();  
    reset_8888();
    init_8888(init_mode);
    while(1){
        while (hk==0);
        while (check_dial_tone()==0);
        dtmf_dialing(number);
        init_8888(cp);
        _wait_ms(60);
        while(to==1){
            if (hk==0){
                sendstring(onhook);
                break;
            }
        }
        _wait_ms(40);
        while (to==1){
            if (hk==0){
                break;
            }
        }
        if (hk==1){
            init_8888(init_mode);
            flag=cek_cp();
            while (flag==10){
                sendstring(ringback);
                sendserial(0x0d);
                flag=cek_cp();
            }
            if (flag==0) sendstring(busy);
            else if (flag==01) sendstring(connect);
            else if (flag==11) sendstring(unknown);
            while (hk==1);
        }
        sendserial(0x0d);        
    }
}
